@@ -14,7 +14,7 @@ class SQLOptions(object):
         self.m_SQL_OptionList.append({"Name": "TIMING", "Value": "OFF", "Comments": '----'})
         self.m_SQL_OptionList.append({"Name": "TIME", "Value": "OFF", "Comments": '----'})
 
-        self.m_SQL_OptionList.append({"Name": "OUTPUT_FORMAT", "Value": "ASCII", "Comments": '----'})
+        self.m_SQL_OptionList.append({"Name": "OUTPUT_FORMAT", "Value": "ASCII", "Comments": 'ASCII|CSV|VERTICAL'})
         self.m_SQL_OptionList.append({"Name": "CSV_HEADER", "Value": "OFF", "Comments": '----'})
         self.m_SQL_OptionList.append({"Name": "CSV_DELIMITER", "Value": ",", "Comments": '----'})
         self.m_SQL_OptionList.append({"Name": "CSV_QUOTECHAR", "Value": "", "Comments": '----'})
@@ -32,6 +32,8 @@ class SQLOptions(object):
         self.m_SQL_OptionList.append({"Name": "CONNURL", "Value": "", "Comments": 'Connection URL'})
         self.m_SQL_OptionList.append({"Name": "CONNPROP", "Value": "", "Comments": 'Connection Props'})
 
+        self.m_SQL_OptionList.append({"Name": "CONN_RETRY_TIMES", "Value": "1", "Comments": 'Connect retry times.'})
+
     def get(self, p_ParameterName):
         """根据参数名称返回参数，若不存在该参数，返回None."""
         for item in self.m_SQL_OptionList:
@@ -63,10 +65,6 @@ class SQLOptions(object):
                     else:
                         m_ParameterValue = p_ParameterDefaultValue
                 self.m_SQL_OptionList[m_nPos]["Value"] = m_ParameterValue
-                if p_ParameterName.upper() == "DEBUG" and m_ParameterValue.upper() == "ON":
-                    os.environ['SQLCLI_DEBUG'] = "1"
-                if p_ParameterName.upper() == "DEBUG" and m_ParameterValue.upper() == "OFF":
-                    del os.environ['SQLCLI_DEBUG']
                 return True
 
         # 对@开头的进行保存和处理
